@@ -1,5 +1,4 @@
-from math import pi
-from pathlib import Path
+from math import pi, atan2
 from typing import Optional, Union, List, Tuple, Iterable
 
 import matplotlib.pyplot as plt
@@ -7,24 +6,31 @@ from matplotlib.axes import Axes
 from matplotlib.collections import PathCollection
 from matplotlib.font_manager import FontProperties
 from matplotlib.patches import Rectangle, RegularPolygon, Circle, Arc, Arrow, \
-    Ellipse, FancyArrow, Patch, FancyArrowPatch
+    Ellipse, FancyArrow, Patch, FancyArrowPatch, FancyBboxPatch, Polygon, Wedge, \
+    BoxStyle
 from matplotlib.path import Path
-from pandas import DataFrame
+from numpy import ndarray, linspace
+from numpy.ma import cos, sin
+from pandas import DataFrame, Series
+from scipy.interpolate import interp1d
 
 from compound_types.arrays import ArrayLike
 from compound_types.built_ins import FloatOrFloatIterable, StrOrStrIterable, \
-    DictOrDictIterable
+    DictOrDictIterable, BoolOrBoolIterable, FloatIterable
+from compound_types.type_checks import all_are_none, one_is_not_none
 from mpl_format.axes.axis_formatter import AxisFormatter
 from mpl_format.axes.axis_utils import new_axes
 from mpl_format.compound_types import FontSize, Color, LegendLocation, \
-    StringMapper
-from mpl_format.io_utils import save_plot
+    StringMapper, ColorOrColorIterable
 from mpl_format.legend.legend_formatter import LegendFormatter
 from mpl_format.patches.patch_list_formatter import PatchListFormatter
 from mpl_format.styles import LINE_STYLE, CAP_STYLE, JOIN_STYLE, ARROW_STYLE, \
-    CONNECTION_STYLE
+    CONNECTION_STYLE, DRAW_STYLE, MARKER_STYLE
 from mpl_format.text.text_formatter import TextFormatter
 from mpl_format.text.text_utils import wrap_text
+from mpl_format.utils.arg_checks import check_h_align
+from mpl_format.utils.color_utils import cross_fade
+from mpl_format.utils.io_utils import save_plot
 
 
 class AxesFormatter(object):
@@ -759,15 +765,7 @@ class AxesFormatter(object):
 
     # region shapes
 
-    @property
-    def rectangles(self) -> PatchListFormatter:
-        """
-        Return a list of the Rectangles on the axes.
-        """
-        return PatchListFormatter([
-            r for r in self._axes.get_children()
-            if isinstance(r, Rectangle)
-        ])
+    # region patches
 
     def add_text(
             self, x: FloatOrFloatIterable,
@@ -808,15 +806,14 @@ class AxesFormatter(object):
         return self
 
     def add_arc(
-            self, x: float, y: float, width: float, height: float,
+            self, x_center: float, y_center: float,
+            width: float, height: float,
             angle: float = 0.0,
             theta_start: float = 0.0, theta_end: float = 360.0,
             alpha: Optional[float] = None,
             cap_style: Optional[Union[str, CAP_STYLE]] = None,
             color: Optional[Color] = None,
             edge_color: Optional[Color] = None,
-            face_color: Optional[Color] = None,
-            fill: bool = True,
             join_style: Optional[Union[str, JOIN_STYLE]] = None,
             label: Optional[str] = None,
             line_style: Optional[Union[str, LINE_STYLE]] = None,
@@ -825,8 +822,8 @@ class AxesFormatter(object):
         """
         Add an elliptical arc, i.e. a segment of an ellipse.
 
-        :param x: The x-coordinate of the center of the ellipse.
-        :param y: The y-coordinate of the center of the ellipse.
+        :param x_center: The x-coordinate of the center of the ellipse.
+        :param y_center: The y-coordinate of the center of the ellipse.
         :param width: The length of the horizontal axis.
         :param height: The length of the vertical axis.
         :param angle: Rotation of the ellipse in degrees (counterclockwise).
@@ -838,8 +835,6 @@ class AxesFormatter(object):
         :param cap_style: Cap style.
         :param color: Use to set both the edge-color and the face-color.
         :param edge_color: Edge color.
-        :param face_color: Face color.
-        :param fill: Whether to fill the rectangle.
         :param join_style: Join style.
         :param label: Label for the object in the legend.
         :param line_style: Line style for edge.
@@ -854,23 +849,24 @@ class AxesFormatter(object):
         # convert args to matplotlib names
         kwargs = {}
         for arg, mpl_arg in zip(
-            [alpha, cap_style, color, edge_color, face_color,
-             fill, join_style, label, line_style, line_width],
-            ['alpha', 'capstyle', 'color', 'edgecolor', 'facecolor',
-             'fill', 'joinstyle', 'label', 'linestyle', 'linewidth']
+            [alpha, cap_style, color, edge_color,
+             join_style, label, line_style, line_width],
+            ['alpha', 'capstyle', 'color', 'edgecolor',
+             'joinstyle', 'label', 'linestyle', 'linewidth']
         ):
             if arg is not None:
                 kwargs[mpl_arg] = arg
         arc = Arc(
-            xy=(x, y), width=width, height=height,
-             angle=angle, theta1=theta_start, theta2=theta_end,
+            xy=(x_center, y_center), width=width, height=height,
+            angle=angle, theta1=theta_start, theta2=theta_end,
             **kwargs
         )
         self._axes.add_artist(arc)
         return self
 
     def add_arrow(
-            self, x: float, y: float, dx: float, dy: float,
+            self, x_tail: float, y_tail: float,
+            dx: float, dy: float,
             width: float = 1.0,
             alpha: Optional[float] = None,
             cap_style: Optional[Union[str, CAP_STYLE]] = None,
@@ -886,8 +882,8 @@ class AxesFormatter(object):
         """
         Add an an arrow patch.
 
-        :param x: The x-coordinate of the arrow tail.
-        :param y: The y-coordinate of the arrow tail.
+        :param x_tail: The x-coordinate of the arrow tail.
+        :param y_tail: The y-coordinate of the arrow tail.
         :param dx: Arrow length in the x direction.
         :param dy: Arrow length in the y direction.
         :param width: Scale factor for the width of the arrow.
@@ -921,14 +917,16 @@ class AxesFormatter(object):
             if arg is not None:
                 kwargs[mpl_arg] = arg
         arrow = Arrow(
-            x=x, y=y, dx=dx, dy=dy, width=width,
+            x=x_tail, y=y_tail, dx=dx, dy=dy, width=width,
             **kwargs
         )
         self._axes.add_artist(arrow)
         return self
 
     def add_circle(
-            self, x: float, y: float,
+            self,
+            x_center: float,
+            y_center: float,
             radius: float,
             alpha: Optional[float] = None,
             cap_style: Optional[Union[str, CAP_STYLE]] = None,
@@ -944,8 +942,8 @@ class AxesFormatter(object):
         """
         Add a rectangle to the Axes.
 
-        :param x: The left rectangle coordinate.
-        :param y: The bottom rectangle coordinate.
+        :param x_center: The left rectangle coordinate.
+        :param y_center: The bottom rectangle coordinate.
         :param radius: The radius of the circle.
         :param alpha: Opacity.
         :param color: Use to set both the edge-color and the face-color.
@@ -975,14 +973,16 @@ class AxesFormatter(object):
             if arg is not None:
                 kwargs[mpl_arg] = arg
         polygon = Circle(
-            xy=(x, y), radius=radius,
+            xy=(x_center, y_center), radius=radius,
             **kwargs
         )
         self._axes.add_artist(polygon)
         return self
 
     def add_ellipse(
-            self, x: float, y: float, width: float, height: float,
+            self,
+            x_center: float, y_center: float,
+            width: float, height: float,
             angle: float = 0.0,
             alpha: Optional[float] = None,
             cap_style: Optional[Union[str, CAP_STYLE]] = None,
@@ -998,8 +998,8 @@ class AxesFormatter(object):
         """
         Add an elliptical arc, i.e. a segment of an ellipse.
 
-        :param x: The x-coordinate of the center of the ellipse.
-        :param y: The y-coordinate of the center of the ellipse.
+        :param x_center: The x-coordinate of the center of the ellipse.
+        :param y_center: The y-coordinate of the center of the ellipse.
         :param width: The length (diameter) of the horizontal axis.
         :param height: The length (diameter) of the vertical axis.
         :param angle: Rotation of the ellipse in degrees (counterclockwise).
@@ -1031,7 +1031,7 @@ class AxesFormatter(object):
             if arg is not None:
                 kwargs[mpl_arg] = arg
         ellipse = Ellipse(
-            xy=(x, y), width=width, height=height, angle=angle,
+            xy=(x_center, y_center), width=width, height=height, angle=angle,
             **kwargs
         )
         self._axes.add_artist(ellipse)
@@ -1210,9 +1210,12 @@ class AxesFormatter(object):
         self._axes.add_artist(arrow)
         return self
 
-    def add_rectangle(
-            self, x: float, y: float, width: float, height: float,
-            angle: float = 0.0,
+    def add_fancy_box_patch(
+            self, x: float, y: float,
+            width: float, height: float,
+            box_style: Union[str, BoxStyle] = 'Round',
+            mutation_scale: Optional[float] = 1,
+            mutation_aspect: Optional[float] = None,
             alpha: Optional[float] = None,
             cap_style: Optional[Union[str, CAP_STYLE]] = None,
             color: Optional[Color] = None,
@@ -1223,16 +1226,21 @@ class AxesFormatter(object):
             label: Optional[str] = None,
             line_style: Optional[Union[str, LINE_STYLE]] = None,
             line_width: Optional[float] = None
-    ) -> 'AxesFormatter':
+    ):
         """
-        Add a rectangle to the Axes.
+        A fancy box around a rectangle with lower left at xy = (x, y)
+        with specified width and height.
 
-        :param x: The left rectangle coordinate.
-        :param y: The bottom rectangle coordinate.
-        :param width: Rectangle width.
-        :param height: Rectangle height.
-        :param angle: Rotation in degrees anti-clockwise about xy
-                      (default is 0.0)
+        :param x: The left coord of the rectangle.
+        :param y: The bottom coord of the rectangle.
+        :param width: The rectangle width.
+        :param height: The rectangle height.
+        :param box_style: The box style.
+        :param mutation_scale: Value with which attributes of arrowstyle
+                               (e.g., head_length) will be scaled.
+        :param mutation_aspect: The height of the rectangle will be squeezed by
+                                this value before the mutation and the mutated
+                                box will be stretched by the inverse of it.
         :param alpha: Opacity.
         :param cap_style: Cap style.
         :param color: Use to set both the edge-color and the face-color.
@@ -1244,6 +1252,149 @@ class AxesFormatter(object):
         :param line_style: Line style for edge.
         :param line_width: Line width for edge.
         """
+        if line_style and isinstance(line_style, LINE_STYLE):
+            line_style = line_style.name
+        if cap_style and isinstance(cap_style, CAP_STYLE):
+            cap_style = cap_style.name
+        if join_style and isinstance(join_style, JOIN_STYLE):
+            join_style = join_style.name
+        # convert args to matplotlib names
+        kwargs = {}
+        for arg, mpl_arg in zip(
+            [alpha, cap_style, color, edge_color, face_color,
+             fill, join_style, label, line_style, line_width],
+            ['alpha', 'capstyle', 'color', 'edgecolor', 'facecolor',
+             'fill', 'joinstyle', 'label', 'linestyle', 'linewidth']
+        ):
+            if arg is not None:
+                kwargs[mpl_arg] = arg
+        fancy_box = FancyBboxPatch(
+            xy=(x, y), width=width, height=height,
+            boxstyle=box_style,
+            mutation_scale=mutation_scale,
+            mutation_aspect=mutation_aspect,
+            **kwargs
+        )
+        self._axes.add_artist(fancy_box)
+        return self
+
+    def add_polygon(
+            self, xy: ndarray, closed: bool = True,
+            alpha: Optional[float] = None,
+            color: Optional[Color] = None,
+            edge_color: Optional[Color] = None,
+            face_color: Optional[Color] = None,
+            fill: bool = True,
+            label: Optional[str] = None,
+            line_style: Optional[Union[str, LINE_STYLE]] = None,
+            line_width: Optional[float] = None,
+            cap_style: Optional[Union[str, CAP_STYLE]] = None,
+            join_style: Optional[Union[str, JOIN_STYLE]] = None
+    ) -> 'AxesFormatter':
+        """
+        Add a general polygon patch.
+
+        :param xy: A numpy array with shape Nx2.
+        :param closed: If True, the polygon will be closed so the starting and
+                       ending points are the same.
+        :param alpha: Opacity.
+        :param cap_style: Cap style.
+        :param color: Use to set both the edge-color and the face-color.
+        :param edge_color: Edge color.
+        :param face_color: Face color.
+        :param fill: Whether to fill the rectangle.
+        :param join_style: Join style.
+        :param label: Label for the object in the legend.
+        :param line_style: Line style for edge.
+        :param line_width: Line width for edge.
+        """
+        if line_style and isinstance(line_style, LINE_STYLE):
+            line_style = line_style.name
+        if cap_style and isinstance(cap_style, CAP_STYLE):
+            cap_style = cap_style.name
+        if join_style and isinstance(join_style, JOIN_STYLE):
+            join_style = join_style.name
+        # convert args to matplotlib names
+        kwargs = {}
+        for arg, mpl_arg in zip(
+            [alpha, cap_style, color, edge_color, face_color,
+             fill, join_style, label, line_style, line_width],
+            ['alpha', 'capstyle', 'color', 'edgecolor', 'facecolor',
+             'fill', 'joinstyle', 'label', 'linestyle', 'linewidth']
+        ):
+            if arg is not None:
+                kwargs[mpl_arg] = arg
+        polygon = Polygon(
+            xy=xy, closed=closed,
+            **kwargs
+        )
+        self._axes.add_artist(polygon)
+        return self
+
+    def add_rectangle(
+            self,
+            width: float, height: float,
+            angle: float = 0.0,
+            x_left: Optional[float] = None, y_bottom: Optional[float] = None,
+            x_center: Optional[float] = None, y_center: Optional[float] = None,
+            alpha: Optional[float] = None,
+            cap_style: Optional[Union[str, CAP_STYLE]] = None,
+            color: Optional[Color] = None,
+            edge_color: Optional[Color] = None,
+            face_color: Optional[Color] = None,
+            fill: bool = True,
+            join_style: Optional[Union[str, JOIN_STYLE]] = None,
+            label: Optional[str] = None,
+            line_style: Optional[Union[str, LINE_STYLE]] = None,
+            line_width: Optional[float] = None
+    ) -> 'AxesFormatter':
+        """
+        Add a rectangle to the Axes.
+
+        :param x_left: The left rectangle coordinate.
+        :param y_bottom: The bottom rectangle coordinate.
+        :param width: Rectangle width.
+        :param height: Rectangle height.
+        :param x_left: The left rectangle coordinate.
+        :param y_bottom: The bottom rectangle coordinate.
+        :param x_center: The center rectangle x-coordinate.
+        :param y_center: The center rectangle y-coordinate.
+        :param angle: Rotation in degrees anti-clockwise about xy (default=0.0)
+        :param alpha: Opacity.
+        :param cap_style: Cap style.
+        :param color: Use to set both the edge-color and the face-color.
+        :param edge_color: Edge color.
+        :param face_color: Face color.
+        :param fill: Whether to fill the rectangle.
+        :param join_style: Join style.
+        :param label: Label for the object in the legend.
+        :param line_style: Line style for edge.
+        :param line_width: Line width for edge.
+        """
+        if not one_is_not_none(x_left, x_center):
+            raise ValueError('Give one of {x_left, x_center}')
+        if not one_is_not_none(y_bottom, y_center):
+            raise ValueError('Give one of {y_bottom, y_center}')
+        if not (
+                all_are_none(x_left, y_bottom) or
+                all_are_none(x_center, y_center)
+        ):
+            raise ValueError(
+                'Give either {x_left, y_bottom} or {x_center, y_center}'
+            )
+        if all_are_none(x_left, y_bottom):
+            x_l = x_center - width / 2
+            y_b = y_center - height / 2
+            r = ((width / 2) ** 2 + (height / 2) ** 2) ** 0.5
+            theta = atan2(height, width)
+            xc_new = x_l + r * cos(theta + pi * angle / 180)
+            yc_new = y_b + r * sin(theta + pi * angle / 180)
+            x = x_l + x_center - xc_new
+            y = y_b + y_center - yc_new
+        else:
+            x = x_left
+            y = y_bottom
+
         if line_style and isinstance(line_style, LINE_STYLE):
             line_style = line_style.name
         if cap_style and isinstance(cap_style, CAP_STYLE):
@@ -1268,7 +1419,9 @@ class AxesFormatter(object):
         return self
 
     def add_regular_polygon(
-            self, x: float, y: float,
+            self,
+            x_center: float,
+            y_center: float,
             num_vertices: int,
             radius: float,
             angle: float = 0,
@@ -1286,8 +1439,8 @@ class AxesFormatter(object):
         """
         Add a rectangle to the Axes.
 
-        :param x: The left rectangle coordinate.
-        :param y: The bottom rectangle coordinate.
+        :param x_center: The x-coordinate of the center of the polygon.
+        :param y_center: The y-coordinate of the center of the polygon.
         :param num_vertices: Number of vertices.
         :param radius: The distance from the center to each of the vertices.
         :param angle: Rotation in degrees anti-clockwise about xy
@@ -1320,13 +1473,486 @@ class AxesFormatter(object):
             if arg is not None:
                 kwargs[mpl_arg] = arg
         polygon = RegularPolygon(
-            xy=(x, y), numVertices=num_vertices, radius=radius,
+            xy=(x_center, y_center), numVertices=num_vertices, radius=radius,
             orientation=pi * angle / 180,
             **kwargs
         )
         self._axes.add_artist(polygon)
         return self
 
+    def add_wedge(
+            self,
+            x_center: float,
+            y_center: float,
+            radius: float,
+            theta_start: float,
+            theta_end: float,
+            width: Optional[float] = None,
+            alpha: Optional[float] = None,
+            cap_style: Optional[Union[str, CAP_STYLE]] = None,
+            color: Optional[Color] = None,
+            edge_color: Optional[Color] = None,
+            face_color: Optional[Color] = None,
+            fill: bool = True,
+            join_style: Optional[Union[str, JOIN_STYLE]] = None,
+            label: Optional[str] = None,
+            line_style: Optional[Union[str, LINE_STYLE]] = None,
+            line_width: Optional[float] = None
+    ):
+        """
+        Add a wedge-shaped patch.
+
+        :param x_center: The x-coordinate of the center of the ellipse.
+        :param y_center: The y-coordinate of the center of the ellipse.
+        :param radius: (Outer) radius.
+        :param theta_start: Starting angle of the arc in degrees.
+                            Relative to angle, e.g. if angle = 45 and
+                            theta_start = 90 the absolute starting angle is 135.
+        :param theta_end: Ending angle of the arc in degrees.
+        :param width: If width is given, then a partial wedge is drawn from
+                      inner radius - width to outer radius.
+        :param alpha: Opacity.
+        :param cap_style: Cap style.
+        :param color: Use to set both the edge-color and the face-color.
+        :param edge_color: Edge color.
+        :param face_color: Face color.
+        :param fill: Whether to fill the rectangle.
+        :param join_style: Join style.
+        :param label: Label for the object in the legend.
+        :param line_style: Line style for edge.
+        :param line_width: Line width for edge.
+        """
+        if line_style and isinstance(line_style, LINE_STYLE):
+            line_style = line_style.name
+        if cap_style and isinstance(cap_style, CAP_STYLE):
+            cap_style = cap_style.name
+        if join_style and isinstance(join_style, JOIN_STYLE):
+            join_style = join_style.name
+        # convert args to matplotlib names
+        kwargs = {}
+        for arg, mpl_arg in zip(
+            [alpha, cap_style, color, edge_color, face_color,
+             fill, join_style, label, line_style, line_width],
+            ['alpha', 'capstyle', 'color', 'edgecolor', 'facecolor',
+             'fill', 'joinstyle', 'label', 'linestyle', 'linewidth']
+        ):
+            if arg is not None:
+                kwargs[mpl_arg] = arg
+        arc = Wedge(
+            center=(x_center, y_center), r=radius,
+            theta1=theta_start, theta2=theta_end,
+            width=width,
+            **kwargs
+        )
+        self._axes.add_artist(arc)
+        return self
+
+    @property
+    def arcs(self) -> PatchListFormatter:
+        """
+        Return a list of the Arcs on the axes.
+        """
+        return PatchListFormatter([
+            a for a in self._axes.get_children()
+            if isinstance(a, Arc)
+        ])
+
+    @property
+    def arrows(self) -> PatchListFormatter:
+        """
+        Return a list of the Arrows on the axes.
+        """
+        return PatchListFormatter([
+            a for a in self._axes.get_children()
+            if isinstance(a, Arrow)
+        ])
+
+    @property
+    def circles(self) -> PatchListFormatter:
+        """
+        Return a list of the Circles on the axes.
+        """
+        return PatchListFormatter([
+            c for c in self._axes.get_children()
+            if isinstance(c, Circle)
+        ])
+
+    @property
+    def ellipses(self) -> PatchListFormatter:
+        """
+        Return a list of the Ellipses on the axes.
+        """
+        return PatchListFormatter([
+            e for e in self._axes.get_children()
+            if isinstance(e, Ellipse)
+        ])
+
+    @property
+    def fancy_arrows(self) -> PatchListFormatter:
+        """
+        Return a list of the FancyArrows on the axes.
+        """
+        return PatchListFormatter([
+            a for a in self._axes.get_children()
+            if isinstance(a, FancyArrow)
+        ])
+
+    @property
+    def fancy_arrow_patches(self) -> PatchListFormatter:
+        """
+        Return a list of the FancyArrowPatches on the axes.
+        """
+        return PatchListFormatter([
+            a for a in self._axes.get_children()
+            if isinstance(a, FancyArrowPatch)
+        ])
+
+    @property
+    def fancy_boxes(self) -> PatchListFormatter:
+        """
+        Return a list of the FancyBoxes on the axes.
+        """
+        return PatchListFormatter([
+            b for b in self._axes.get_children()
+            if isinstance(b, FancyBboxPatch)
+        ])
+
+    @property
+    def polygons(self) -> PatchListFormatter:
+        """
+        Return a list of the Polygons on the axes.
+        """
+        return PatchListFormatter([
+            p for p in self._axes.get_children()
+            if isinstance(p, Polygon)
+        ])
+
+    @property
+    def rectangles(self) -> PatchListFormatter:
+        """
+        Return a list of the Rectangles on the axes.
+        """
+        return PatchListFormatter([
+            r for r in self._axes.get_children()
+            if isinstance(r, Rectangle)
+        ])
+
+    @property
+    def regular_polygons(self) -> PatchListFormatter:
+        """
+        Return a list of the RegularPolygons on the axes.
+        """
+        return PatchListFormatter([
+            r for r in self._axes.get_children()
+            if isinstance(r, RegularPolygon)
+        ])
+
+    @property
+    def wedges(self) -> PatchListFormatter:
+        """
+        Return a list of the Wedges on the axes.
+        """
+        return PatchListFormatter([
+            w for w in self._axes.get_children()
+            if isinstance(w, Wedge)
+        ])
+
+    # endregion
+
+    # region custom shapes
+
+    def add_v_density(
+            self, x: float,
+            y_to_z: Series,
+            color: Color,
+            color_min: Optional[Color] = None,
+            width: float = 0.8,
+            z_max: Optional[float] = None,
+            h_align: str = 'center'
+    ) -> 'AxesFormatter':
+        """
+        Add a vertical density bar to the plot.
+
+        :param x: The x-coordinate of the bar.
+        :param y_to_z: A mapping of the bar's y-coordinate to it density.
+        :param color: The color of the density bar.
+        :param color_min: Optional 2nd color to fade out to.
+        :param width: The bar width.
+        :param z_max: Value to scale down densities by to get to a range of
+                      0 to 1. Defaults to max value of y_to_z.
+        :param h_align: Horizontal alignment. One of {'left', 'center', 'right'}
+        """
+        check_h_align(h_align)
+
+        if z_max is None:
+            z_max = y_to_z.max()
+        y = y_to_z.index.to_list()
+        y_lowers = y[: -1]
+        y_uppers = y[1:]
+
+        if h_align == 'left':
+            x_p = x
+        elif h_align == 'center':
+            x_p = x - width / 2
+        else:
+            x_p = x - width
+
+        alphas = (y_to_z / z_max).rolling(2).mean().shift(-1).dropna()
+
+        if color_min is None:
+            color_min = color
+        colors = cross_fade(from_color=color_min, to_color=color, amount=alphas)
+
+        for y_lower, y_upper, alpha, color in zip(
+                y_lowers, y_uppers, alphas, colors
+        ):
+            self.add_rectangle(
+                x=x_p, y=y_lower,
+                width=width, height=y_upper - y_lower,
+                face_color=color, alpha=alpha
+            )
+
+        return self
+
+    # endregion
+
+    # region plots
+
+    def add_v_bars(
+            self,
+            x: Union[str, FloatOrFloatIterable],
+            y: Union[str, FloatOrFloatIterable],
+            width: Union[str, FloatOrFloatIterable],
+            box_style: Union[BoxStyle, Iterable[BoxStyle]],
+            data: Optional[DataFrame] = None,
+            y_0: Optional[Union[str, FloatOrFloatIterable]] = 0.0,
+            h_align: str = 'center',
+            mutation_scale: Union[str, FloatOrFloatIterable] = 1,
+            mutation_aspect: Optional[Union[str, FloatOrFloatIterable]] = None,
+            alpha: Optional[Union[str, FloatOrFloatIterable]] = None,
+            cap_style: Optional[Union[
+                StrOrStrIterable, CAP_STYLE, Iterable[CAP_STYLE]
+            ]] = None,
+            color: Optional[ColorOrColorIterable] = None,
+            edge_color: Optional[ColorOrColorIterable] = None,
+            face_color: Optional[ColorOrColorIterable] = None,
+            fill: BoolOrBoolIterable = True,
+            line_style: Optional[Union[
+                StrOrStrIterable, LINE_STYLE, Iterable[LINE_STYLE]
+            ]] = None,
+            line_width: Optional[FloatOrFloatIterable] = None
+    ) -> 'AxesFormatter':
+        """
+        Add vertical bars to the plot with the given BoxStyle.
+
+        :param x: X-coordinate for each bar, or name of column.
+        :param y: Y-coordinate of top of each bar, or name of column.
+        :param width: Width of each bar, or name of column.
+        :param box_style: The style of each box, or name of column.
+        :param data: Optional DataFrame to extract sequences of attributes.
+        :param y_0: Y-coordinate of bottom of each bar. Defaults to 0.
+        :param h_align: Horizontal alignment. One of {'left', 'center', 'right'}
+        :param mutation_scale: Scaling factor applied to the attributes of the
+                               box style (e.g. pad or rounding_size).
+        :param mutation_aspect: The height of the rectangle will be squeezed by
+                                this value before the mutation and the mutated
+                                box will be stretched by the inverse of it.
+                                For example, this allows different horizontal
+                                and vertical padding.
+        :param alpha: Opacity from 0 to 1.
+        :param cap_style: One of {'butt', 'round', 'projecting'}
+        :param color: Color for faces and edges.
+        :param edge_color: Color for edges.
+        :param face_color: Color for faces.
+        :param fill: Whether bars are filled.
+        :param line_style: Line style.
+        :param line_width: Line width.
+        """
+        check_h_align(h_align)
+
+        def get_data(item) -> Series:
+            if data is None:
+                return item
+            if isinstance(item, str) and item in data.columns:
+                return data[item]
+            else:
+                return item
+
+        def make_iterable(item):
+            if not isinstance(item, Iterable) or isinstance(item, str):
+                item = [item] * len(x)
+            return item
+
+        x = get_data(x)
+        y = make_iterable(get_data(y))
+        y_0 = make_iterable(get_data(y_0))
+        width = make_iterable(get_data(width))
+        box_style = make_iterable(get_data(box_style))
+        mutation_scale = make_iterable(get_data(mutation_scale))
+        mutation_aspect = make_iterable(get_data(mutation_aspect))
+        alpha = make_iterable(get_data(alpha))
+        cap_style = make_iterable(get_data(cap_style))
+        color = make_iterable(get_data(color))
+        edge_color = make_iterable(get_data(edge_color))
+        face_color = make_iterable(get_data(face_color))
+        fill = make_iterable(get_data(fill))
+        line_style = make_iterable(get_data(line_style))
+        line_width = make_iterable(get_data(line_width))
+
+        for (
+                x_i, y_i, y_0_i, w_i, bs_i,
+                ms_i, ma_i, a_i, cs_i,
+                c_i, ec_i, fc_i,
+                f_i, ls_i, lw_i
+        ) in zip(
+            x, y, y_0, width, box_style,
+            mutation_scale, mutation_aspect, alpha, cap_style,
+            color, edge_color, face_color,
+            fill, line_style, line_width
+        ):
+            if h_align == 'left':
+                x_p = x_i
+            elif h_align == 'center':
+                x_p = x_i - w_i / 2
+            else:
+                x_p = x_i - width
+
+            self.add_fancy_box_patch(
+                x=x_p, y=y_0_i, width=w_i, height=y_i - y_0_i,
+                box_style=bs_i,
+                mutation_scale=ms_i, mutation_aspect=ma_i,
+                alpha=a_i, cap_style=cs_i,
+                color=c_i, edge_color=ec_i, face_color=fc_i,
+                fill=f_i, line_style=ls_i, line_width=lw_i
+            )
+
+        return self
+
+    def add_v_pills(
+            self,
+            x: Union[str, FloatOrFloatIterable],
+            y: Union[str, FloatOrFloatIterable],
+            width: float,
+            data: Optional[DataFrame] = None,
+            y_0: Optional[Union[str, FloatOrFloatIterable]] = 0.0,
+            h_align: str = 'center',
+            alpha: Optional[Union[str, FloatOrFloatIterable]] = None,
+            color: Optional[ColorOrColorIterable] = None,
+            edge_color: Optional[ColorOrColorIterable] = None,
+            face_color: Optional[ColorOrColorIterable] = None,
+            fill: BoolOrBoolIterable = True,
+            line_style: Optional[Union[
+                StrOrStrIterable, LINE_STYLE, Iterable[LINE_STYLE]
+            ]] = None,
+            line_width: Optional[FloatOrFloatIterable] = None
+    ) -> 'AxesFormatter':
+        """
+        Add vertical bars to the plot with the given BoxStyle.
+
+        :param x: X-coordinate for each bar, or name of column.
+        :param y: Y-coordinate of top of each bar, or name of column.
+        :param width: Width of each bar, or name of column.
+        :param data: Optional DataFrame to extract sequences of attributes.
+        :param y_0: Y-coordinate of bottom of each bar. Defaults to 0.
+        :param h_align: One of {'left', 'center', 'right'}.
+        :param alpha: Opacity from 0 to 1.
+        :param color: Color for faces and edges.
+        :param edge_color: Color for edges.
+        :param face_color: Color for faces.
+        :param fill: Whether bars are filled.
+        :param line_style: Line style.
+        :param line_width: Line width.
+        """
+        mutation_aspect = (
+                (self.width() / self.height()) *
+                (self.get_y_height() / self.get_x_width())
+        )
+        self.add_v_bars(
+            data=data, x=x, y=y, width=width,
+            box_style=BoxStyle.Round(pad=0.0, rounding_size=width / 2),
+            y_0=y_0, h_align=h_align,
+            mutation_aspect=mutation_aspect,
+            alpha=alpha,
+            color=color, edge_color=edge_color, face_color=face_color,
+            fill=fill, line_style=line_style, line_width=line_width
+        )
+        return self
+
+    def add_line(
+            self,
+            x: FloatIterable, y: FloatIterable,
+            smooth: Union[bool, int] = False, smooth_order: int = 2,
+            alpha: Optional[float] = None,
+            color: Optional[Color] = None,
+            draw_style: Optional[Union[str, DRAW_STYLE]] = None,
+            label: Optional[str] = None,
+            line_style: Optional[Union[str, LINE_STYLE]] = None,
+            line_width: Optional[float] = None,
+            marker: Optional[Union[str, MARKER_STYLE]] = None,
+            marker_edge_color: Optional[Color] = None,
+            marker_edge_width: Optional[float] = None,
+            marker_face_color: Optional[Color] = None,
+            marker_face_color_alt: Optional[Color] = None,
+            marker_size: Optional[float] = None
+    ) -> 'AxesFormatter':
+        """
+        Add a line to the plot.
+
+        :param x: X-coordinates of the line.
+        :param y: Y-coordinates of the line.
+        :param smooth: True or False or number of points. True -> 1,000 points.
+        :param smooth_order: Order for smoothing e.g. 2 = quadratic, 3 = cubic
+        :param alpha: Opacity of the line.
+        :param color: Color of the line.
+        :param draw_style: Draw style. One of {'default', 'steps', 'steps-pre',
+                           'steps-mid', 'steps-post'}
+        :param label: Label.
+        :param line_style: Line Style. One of {'-', '--', '-.', ':', '',
+                           (offset, on-off-seq), ...}
+        :param line_width: Line width.
+        :param marker: Marker style.
+        :param marker_edge_color: Marker edge color.
+        :param marker_edge_width: Marker edge width.
+        :param marker_face_color: Marker face color.
+        :param marker_face_color_alt: Alt marker face color.
+        :param marker_size: Marker size.
+        """
+        if smooth is not False:
+            if smooth is True:
+                smooth = 1000
+            f_smooth = interp1d(x, y, kind=smooth_order)
+            x_smooth = linspace(min(x), max(x), smooth)
+            y_smooth = f_smooth(x_smooth)
+            x = x_smooth
+            y = y_smooth
+        if draw_style is not None:
+            draw_style = (
+                draw_style if isinstance(draw_style, str)
+                else DRAW_STYLE.get_name(draw_style)
+            )
+        if line_style is not None:
+            line_style = (
+                line_style if isinstance(line_style, str)
+                else LINE_STYLE.get_name(line_style)
+            )
+
+        self._axes.plot(
+            x, y, alpha=alpha, color=color,
+            drawstyle=draw_style,
+            label=label,
+            ls=line_style,
+            lw=line_width,
+            marker=marker,
+            markersize=marker_size,
+            mec=marker_edge_color,
+            mew=marker_edge_width,
+            mfc=marker_face_color,
+            mfcalt=marker_face_color_alt
+        )
+        return self
+
+    # endregion
+
     # endregion
 
     def set_title_font_family(self, font_name: str) -> 'AxesFormatter':
@@ -1386,6 +2012,14 @@ class AxesFormatter(object):
         """
         return self.get_x_lim()[1]
 
+    def get_x_width(self) -> float:
+
+        return abs(self.get_x_max() - self.get_x_min())
+
+    def get_y_height(self) -> float:
+
+        return abs(self.get_y_max() - self.get_y_min())
+
     def set_x_min(self, left: float = None) -> 'AxesFormatter':
         """
         Set the x-axis lower view limit.
@@ -1426,6 +2060,32 @@ class AxesFormatter(object):
         self.set_y_lim(None, right)
         return self
 
+    def width(self, units: str = 'inches') -> float:
+
+        if units not in ('inches', 'pixels'):
+            raise ValueError("units not in ('inches', 'pixels')")
+        fig = self._axes.figure
+        bbox = self.axes.get_window_extent().transformed(
+            fig.dpi_scale_trans.inverted()
+        )
+        width = bbox.width
+        if units == 'pixels':
+            width *= fig.dpi
+        return width
+
+    def height(self, units: str = 'inches') -> float:
+
+        if units not in ('inches', 'pixels'):
+            raise ValueError("units not in ('inches', 'pixels')")
+        fig = self._axes.figure
+        bbox = self.axes.get_window_extent().transformed(
+            fig.dpi_scale_trans.inverted()
+        )
+        height = bbox.height
+        if units == 'pixels':
+            height *= fig.dpi
+        return height
+
     # endregion
 
     def grid(self, value: bool = True,
