@@ -2,7 +2,7 @@ import matplotlib.pyplot as plt
 from matplotlib.axis import Axis
 from matplotlib.lines import Line2D
 from matplotlib.text import Text
-from matplotlib.ticker import StrMethodFormatter, FuncFormatter
+from matplotlib.ticker import FuncFormatter
 
 from mpl_format.compound_types import FontSize, Color, StringMapper
 from mpl_format.text.text_formatter import TextFormatter
@@ -160,25 +160,63 @@ class AxisFormatter(object):
 
         :param max_width: The maximum character width per line.
         """
+        tick_labels = self._axis.get_ticklabels()
+        if all(t.get_text() == '' for t in tick_labels):
+            return self  # non string tick-labels e.g. line plot
         self._axis.set_ticklabels([
-            wrap_text(text, max_width)
-            for text in self._axis.get_ticklabels()
+            wrap_text(text, max_width) for text in tick_labels
         ])
         return self
 
-    def set_format_integer(self, separator: str = ',') -> 'AxisFormatter':
+    def set_format_integer(self,
+                           categorical: bool = False,
+                           kmbt: bool = False) -> 'AxisFormatter':
         """
         Format an axis with currency symbols and separators.
 
-        :param separator: The symbol to separate each group of 3 digits.
+        :param categorical: Whether the axis is displaying categorical items
+                            e.g. for bar plots.
+        :param kmbt: Whether to abbreviate numbers using K, M, B and T for
+                     thousands, millions, billions and trillions.
         """
-        fmt = '{x:%s.0f}' % separator
-        tick = StrMethodFormatter(fmt)
-        self._axis.set_major_formatter(tick)
+        def to_integer(text: Text):
+            if isinstance(text, Text):
+                t = text.get_text()
+            else:
+                t = str(text)
+            if t == '':
+                return ''
+            number = float(t)
+            if not kmbt:
+                return f'{int(number):,}'
+            else:
+                for power, abbrev in zip(
+                        [12, 9, 6, 3],
+                        ['T', 'B', 'M', 'K']
+                ):
+                    if number >= 10 ** power:
+                        num = number / 10 ** power
+                        if num == int(num):
+                            num = int(num)
+                        return f'{num:,}{abbrev}'
+                if number == int(number):
+                    number = int(number)
+                return f'{number:,}'
+
+        if not categorical:
+            tick = FuncFormatter(lambda x, pos: to_integer(x))
+            self._axis.set_major_formatter(tick)
+        else:
+            self._axis.set_ticklabels([
+                to_integer(text) for text in self._axis.get_ticklabels()
+            ])
+
         return self
 
     def set_format_currency(
-            self, symbol: str = '$', num_decimals: int = 0
+            self, symbol: str = '$', num_decimals: int = 0,
+            categorical: bool = False,
+            kmbt: bool = False
     ) -> 'AxisFormatter':
         """
         Format an axis with currency symbols and separators.
@@ -186,28 +224,78 @@ class AxisFormatter(object):
         :param symbol: The currency symbol to use.
         :param num_decimals: The number of decimal places to use
                              (typically 0 or 2).
+        :param categorical: Whether the axis is displaying categorical items
+                            e.g. for bar plots.
+        :param kmbt: Whether to abbreviate numbers using K, M, B and T for
+                     thousands, millions, billions and trillions.
         """
-        fmt = '%s{x:,.%sf}' % (symbol, num_decimals)
-        tick = StrMethodFormatter(fmt)
-        self._axis.set_major_formatter(tick)
+
+        def to_currency(text: Text):
+            if isinstance(text, Text):
+                t = text.get_text()
+            else:
+                t = str(text)
+            if t == '':
+                return ''
+            number = float(t)
+            if not kmbt:
+                return f'{symbol}{number:,.{num_decimals}f}'
+            else:
+                for power, abbrev in zip(
+                        [12, 9, 6, 3],
+                        ['T', 'B', 'M', 'K']
+                ):
+                    if number >= 10 ** power:
+                        num = number / 10 ** power
+                        if num == int(num):
+                            num = int(num)
+                        return f'{symbol}{num:,}{abbrev}'
+                if number == int(number):
+                    number = int(number)
+                return f'{symbol}{number:,.{num_decimals}f}'
+
+        if not categorical:
+            tick = FuncFormatter(lambda x, pos: to_currency(x))
+            self._axis.set_major_formatter(tick)
+        else:
+            self._axis.set_ticklabels([
+                to_currency(text) for text in self._axis.get_ticklabels()
+            ])
+
         return self
 
     def set_format_percent(self, num_decimals: int = 0,
-                           multiply_100: bool = True) -> 'AxisFormatter':
+                           multiply_100: bool = True,
+                           categorical: bool = False) -> 'AxisFormatter':
         """
         Format an axis as a percentage.
 
         :param num_decimals: Number of decimal places for the resulting label.
         :param multiply_100: Whether to multiply the existing value by 100
                              before formatting.
+        :param categorical: Whether the axis is displaying categorical items
+                            e.g. for bar plots.
         """
-        def percent_formatter(s, _):
-            s = float(s)
-            if multiply_100:
-                s *= 100
-            return f'{s:.{num_decimals}f}%'
+        if not categorical:
+            def percent_formatter(value, pos):
+                value = float(value)
+                if multiply_100:
+                    value *= 100
+                return f'{value:.{num_decimals}f}%'
+            self._axis.set_major_formatter(FuncFormatter(percent_formatter))
+        else:
+            def to_percent(text: Text):
+                t = text.get_text()
+                if t == '':
+                    return ''
+                number = float(t)
+                if multiply_100:
+                    number *= 100
+                return f'{number:.{num_decimals}f}%'
+            self._axis.set_ticklabels([
+                to_percent(text) for text in self._axis.get_ticklabels()
+            ])
 
-        self._axis.set_major_formatter(FuncFormatter(percent_formatter))
         return self
 
     def set_tick_label_size(self, font_size: FontSize) -> 'AxisFormatter':
