@@ -5,12 +5,14 @@ import matplotlib.pyplot as plt
 from matplotlib.axes import Axes
 from matplotlib.figure import Figure
 from numpy import ndarray, reshape
+from numpy.ma import empty_like
 
+from mpl_format.axes import AxisFormatter
 from mpl_format.axes.axes_formatter import AxesFormatter
 from mpl_format.axes.axes_formatter_array import AxesFormatterArray
+from mpl_format.axes.axis_formatter_array import AxisFormatterArray
 from mpl_format.compound_types import StringMapper
-from mpl_format.io_utils import save_plot
-
+from mpl_format.utils.io_utils import save_plot
 
 TextSetter = TypeVar(
     'TextSetter',
@@ -75,6 +77,46 @@ class FigureFormatter(object):
         else:
             return AxesFormatterArray(self._axes)
 
+    @property
+    def x_axes(self) -> Union[AxisFormatter, AxisFormatterArray]:
+        """
+        Return an AxisFormatter or AxisFormatterArray for the X-Axis or X-Axes
+        of the wrapped Axes.
+        """
+        if not self._has_array:
+            return AxesFormatter(self._axes).x_axis
+        else:
+            axes = empty_like(self._axes, dtype=AxisFormatter)
+            if axes.ndim == 1:
+                for i in range(self._axes.shape[0]):
+                    axes[i] = AxisFormatter(self._axes[i].xaxis)
+            elif axes.ndim == 2:
+                for i in range(axes.shape[0]):
+                    for j in range(axes.shape[1]):
+                        axes[i, j] = AxisFormatter(self._axes[i, j].xaxis)
+
+        return AxisFormatterArray(axes)
+
+    @property
+    def y_axes(self) -> Union[AxisFormatter, AxisFormatterArray]:
+        """
+        Return an AxisFormatter or AxisFormatterArray for the Y-Axis or Y-Axes
+        of the wrapped Axes.
+        """
+        if not self._has_array:
+            return AxesFormatter(self._axes).y_axis
+        else:
+            axes = empty_like(self._axes, dtype=AxisFormatter)
+            if axes.ndim == 1:
+                for i in range(self._axes.shape[0]):
+                    axes[i] = AxisFormatter(self._axes[i].yaxis)
+            elif axes.ndim == 2:
+                for i in range(axes.shape[0]):
+                    for j in range(axes.shape[1]):
+                        axes[i, j] = AxisFormatter(self._axes[i, j].yaxis)
+
+        return AxisFormatterArray(axes)
+
     @property
     def figure(self) -> Figure:
         return self._figure
@@ -591,13 +633,13 @@ class FigureFormatter(object):
         Adjust the padding of the Figure.
 
         :param left: The position of the left edge of the subplots,
-                     as a fraction of the figure width.
+                     as a fraction of the figure width. Use 0 for no margin.
         :param right: The position of the right edge of the subplots,
-                      as a fraction of the figure width.
+                      as a fraction of the figure width. Use 1 for no margin.
         :param top: The position of the top edge of the subplots,
-                    as a fraction of the figure height.
+                    as a fraction of the figure height. Use 1 for no margin.
         :param bottom: The position of the bottom edge of the subplots,
-                       as a fraction of the figure height.
+                       as a fraction of the figure height. Use 0 for no margin.
         :param w_space: The width of the padding between subplots,
                         as a fraction of the average axes width.
         :param h_space: The height of the padding between subplots,
@@ -610,6 +652,90 @@ class FigureFormatter(object):
         )
         return self
 
+    def set_left_margin(self, margin_size: float) -> 'FigureFormatter':
+        """
+        Set the margin size at the left of the plot.
+
+        :param margin_size: Value between 0 and 1. 0 for no margin
+        """
+        self.subplots_adjust(left=margin_size)
+        return self
+
+    def set_right_margin(self, margin_size: float) -> 'FigureFormatter':
+        """
+        Set the margin size at the right of the plot.
+
+        :param margin_size: Value between 0 and 1. 0 for no margin
+        """
+        self.subplots_adjust(right=1 - margin_size)
+        return self
+
+    def set_top_margin(self, margin_size: float) -> 'FigureFormatter':
+        """
+        Set the margin size at the top of the plot.
+
+        :param margin_size: Value between 0 and 1. 0 for no margin
+        """
+        self.subplots_adjust(top=1 - margin_size)
+        return self
+
+    def set_bottom_margin(self, margin_size: float) -> 'FigureFormatter':
+        """
+        Set the margin size at the bottom of the plot.
+
+        :param margin_size: Value between 0 and 1. 0 for no margin
+        """
+        self.subplots_adjust(bottom=margin_size)
+        return self
+
+    def set_horizontal_margin(self, margin_size: float) -> 'FigureFormatter':
+        """
+        Set the margin size at the left and right of the plot.
+
+        :param margin_size: Value between 0 and 1. 0 for no margin
+        """
+        self.subplots_adjust(left=margin_size, right=1 - margin_size)
+        return self
+
+    def set_vertical_margin(self, margin_size: float) -> 'FigureFormatter':
+        """
+        Set the margin size at the top and bottom of the plot.
+
+        :param margin_size: Value between 0 and 1. 0 for no margin
+        """
+        self.subplots_adjust(bottom=margin_size, top=1 - margin_size)
+        return self
+
+    def set_margin(self, margin_size: float) -> 'FigureFormatter':
+        """
+        Set the margin size at the top of the plot.
+
+        :param margin_size: Value between 0 and 1. 0 for no margin
+        """
+        self.subplots_adjust(bottom=margin_size, top=1 - margin_size,
+                             left=margin_size, right=1 - margin_size)
+        return self
+
+    def set_horizontal_spacing(self, spacing: float) -> 'FigureFormatter':
+        """
+        Set the horizontal spacing between each subplot.
+
+        :param spacing: The width of the padding between subplots,
+                        as a fraction of the average axes width.
+        """
+        self.subplots_adjust(w_space=spacing)
+        return self
+
+    def set_vertical_spacing(self, spacing: float) -> 'FigureFormatter':
+        """
+        Set the vertical spacing between each subplot.
+
+        :param spacing: The height of the padding between subplots,
+                        as a fraction of the average axes height.
+        """
+        self.subplots_adjust(h_space=spacing)
+        return self
+
     def tight_layout(self) -> 'FigureFormatter':
         """
         Call the tight_layout method on the wrapped Figure.
